import os
import glob
import unittest
import importlib
import functools
import tempfile

here = os.path.dirname(__file__)

def with_test_file(method):
    @functools.wraps(method)
    def _wrapped(*args, **kwargs):
        f = tempfile.NamedTemporaryFile(suffix='.bloom')
        kwargs['filename'] = f.name
        try:
            return method(*args, **kwargs)
        finally:
            f.close()
    return _wrapped

def test_all():
    suite = unittest.TestSuite()
    for fname in glob.glob(os.path.join(here, '*.py')):
        if '__init__' in fname:
            continue
        module = importlib.import_module('tests.' + os.path.basename(fname).split('.py')[0])
        if hasattr(module, 'suite'):
            suite.addTest(module.suite())
    return suite



import random
import string
import unittest
import tempfile

import pybloomfilter

from tests import with_test_file

class AccuracyTestCase(unittest.TestCase):
    def test_strings(self):
        random_strings = list(set(''.join(random.choice(string.lowercase + string.uppercase)
                                          for _ in range(6)) for _ in range(10000)))
        random.shuffle(random_strings)

        for accuracy in (0.1, 0.01, 0.001):
            bf = pybloomfilter.BloomFilter(8000, accuracy)
            bf.update(random_strings[:8000])
            false_pos, false_neg = 0, 0
            for test in random_strings[8000:10000]:
                if test in bf:
                    false_pos += 1
            for test in random_strings[6000:8000]:
                if test not in bf:
                    false_neg += 1
            false_pos_rate = false_pos / 2000.
            false_neg_rate = false_neg / 2000.
            self.assertTrue(false_pos_rate <= accuracy*2, "accuracy fail: %0.4f > %0.4f" % (false_pos_rate, accuracy))
            self.assertEqual(false_neg_rate, 0.0, "false negative rate is nonzero: %0.4f" % (false_neg_rate,))
            del bf
            print false_pos_rate, accuracy

    def test_ints(self):
        random_strings = list(range(10000))

        for accuracy in (0.1, 0.01, 0.001):
            bf = pybloomfilter.BloomFilter(8000, accuracy)
            bf.update(random_strings[:8000])
            false_pos, false_neg = 0, 0
            for test in random_strings[8000:10000]:
                if test in bf:
                    false_pos += 1
            for test in random_strings[6000:8000]:
                if test not in bf:
                    false_neg += 1
            false_pos_rate = false_pos / 2000.
            false_neg_rate = false_neg / 2000.
            self.assertTrue(false_pos_rate <= accuracy*7, "accuracy fail: %0.4f > %0.4f" % (false_pos_rate, accuracy))
            self.assertEqual(false_neg_rate, 0.0, "false negative rate is nonzero: %0.4f" % (false_neg_rate,))
            print false_pos_rate, accuracy
            del bf

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AccuracyTestCase))
    return suite



import os
import sys

here = os.path.dirname(__file__)

ext_files = ["src/mmapbitarray.c",
             "src/bloomfilter.c",
             "src/md5.c",
             "src/primetester.c",
             ]

kwargs = {}

try:
    import Cython
    sys.path.insert(0, os.path.join(here, 'fake_pyrex'))
except ImportError:
    pass


from setuptools import setup, Extension

try:
    from Cython.Distutils import build_ext
    print "info: Building from Cython"
    ext_files.append("src/pybloomfilter.pyx")
    kwargs['cmdclass'] = {'build_ext': build_ext}
    try:
        if 'sdist' not in sys.args:
            os.unlink(os.path.join(here, 'src', 'pybloomfilter.c'))
            os.unlink(os.path.join(here, 'pybloomfilter.so'))
    except:
        pass
except ImportError:
    if '--cython' in sys.argv:
        raise
    ext_files.append("src/pybloomfilter.c")
    print "info: Building from C"

if '--cython' in sys.argv:
    sys.argv.remove('--cython')

ext_modules = [Extension("pybloomfilter",
                         ext_files,
                         libraries=['crypto'])]

requirements = []

if sys.version_info[0] < 3 and sys.version_info[1] < 7:
    requirements.append('importlib')

setup(
  name = 'pybloomfiltermmap',
  version = "0.3.10",
  author = "Michael Axiak, Rob Stacey",
  author_email = "mike@axiak.net",
  url = "http://github.com/axiak/pybloomfiltermmap/",
  description = "A Bloom filter (bloomfilter) for Python built on mmap",
  license = "MIT License",
  test_suite = 'tests.test_all',
  install_requires=requirements,
  ext_modules = ext_modules,
  classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: C',
        'Programming Language :: Cython',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
  **kwargs
)




import sys
import unittest
import tempfile

import pybloomfilter

from tests import with_test_file

class SimpleTestCase(unittest.TestCase):
    @with_test_file
    def test_number(self, filename):
        bf = pybloomfilter.BloomFilter(100, 0.01, filename)
        bf.add(1234)
        self.assertEquals(1234 in bf, True)

    @with_test_file
    def test_string(self, filename):
        bf = pybloomfilter.BloomFilter(100, 0.01, filename)
        bf.add("test")
        self.assertEquals("test" in bf, True)

    @with_test_file
    def test_others(self, filename):
        bf = pybloomfilter.BloomFilter(100, 0.01, filename)
        for elem in (1.2, 2343L, (1, 2), object(), u'\u2131\u3184'):
            bf.add(elem)
            self.assertEquals(elem in bf, True)

    def test_number_nofile(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        bf.add(1234)
        self.assertEquals(1234 in bf, True)

    def test_string_nofile(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        bf.add("test")
        self.assertEquals("test" in bf, True)

    def test_others_nofile(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        for elem in (1.2, 2343L, (1, 2), object(), u'\u2131\u3184'):
            bf.add(elem)
            self.assertEquals(elem in bf, True)

    #@unittest.skip("unfortunately large files cannot be tested on Travis")
    @with_test_file
    def _test_large_file(self, filename):
        bf = pybloomfilter.BloomFilter(400000000, 0.01, filename)
        bf.add(1234)
        self.assertEquals(1234 in bf, True)

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SimpleTestCase))
    return suite
